/*******************************************************************************
 * $Id: GappedAlignment.cpp,v 1.27 2004/03/01 02:40:08 darling Exp $
 * This file is copyright 2002-2004 Aaron Darling.  All rights reserved.
 * Please see the file called COPYING for licensing, copying, and modification
 * rights.  Redistribution of this file, in whole or in part is prohibited
 * without express permission.
 ******************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "libMems/GappedAlignment.h"
#include <sstream>
#include "libGenome/gnFilter.h"

#include <fstream>


GappedAlignment::GappedAlignment() : AbstractMatch(){
	align_length = 0;
	score = 0;
}

GappedAlignment::GappedAlignment( uint seq_count, gnSeqI align_length ) : AbstractMatch( seq_count )
{
	for( uint seqI = 0; seqI < seq_count; seqI++ ){
		string empty_str;
		align_matrix.push_back( empty_str );
		length.push_back( 0 );
	}
	this->align_length = align_length;
	score = 0;
	identity = NumericMatrix< double >( seq_count, seq_count );
	identity.init( 0 );
}

GappedAlignment::GappedAlignment( const GappedAlignment& cr )
{
	*this = cr;
}

GappedAlignment::~GappedAlignment(){}

GappedAlignment& GappedAlignment::operator=( const GappedAlignment& cr )
{
	AbstractMatch::operator=( cr );
	align_length = cr.align_length;
	score = cr.score;
	align_matrix = cr.align_matrix;
	identity = cr.identity;
	length = cr.length;
	return *this;
}

void GappedAlignment::setAlignment( vector< string >& seq_align, double score ){
	align_matrix = seq_align;
	align_length = align_matrix[ 0 ].length();
	this->score = score;
}

double GappedAlignment::getScore() const{
	return score;
}

string GappedAlignment::getColumn( gnSeqI colI ) const{
	string column;
	return column;
}

double GappedAlignment::getIdentity( uint seqI, uint seqJ ) const{
	return identity( seqI, seqJ );
}

std::ostream& operator<<( std::ostream& os, const GappedAlignment& cr ); //write to source.
std::ostream& operator<<( std::ostream& os, const GappedAlignment& cr ){
	os << "ClustalResultSeqs: " << cr.SeqCount() << endl;
	os << "ClustalResultScore: " << cr.score << endl;
	os << cr.align_length;
	for( uint seqI = 0; seqI < cr.SeqCount(); seqI++ )
		os << '\t' << cr.m_start[ seqI ];
	os << endl;
	for( uint seqI = 0; seqI < cr.SeqCount(); seqI++ ){
		os << cr.align_matrix[ seqI ] << endl;
	}
	return os;
};

std::istream& operator>>( std::istream& is, GappedAlignment& cr ); // read from source
std::istream& operator>>( std::istream& is, GappedAlignment& cr ){
	uint seq_count;
	string nuffin;
	is >> nuffin;
	is >> seq_count;
	is >> nuffin;
	is >> cr.score;
	cr = GappedAlignment( seq_count, 0 );
	for( uint seqI = 0; seqI < seq_count; seqI++ ){
		int64 startI;
		is >> startI;
		cr.SetStart( seqI, startI );
	}
	for( uint seqI = 0; seqI < seq_count; seqI++ ){
		string seq;
		is >> seq;
		cr.align_matrix.push_back( seq );
	}
	cr.align_length = cr.align_matrix[ 0 ].length();
	return is;
};

