/*******************************************************************************
 * $Id: MuscleInterface.cpp,v 1.27 2004/03/01 02:40:08 darling Exp $
 * This file is copyright 2002-2004 Aaron Darling.  All rights reserved.
 * Please see the file called COPYING for licensing, copying, and modification
 * rights.  Redistribution of this file, in whole or in part is prohibited
 * without express permission.
 ******************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".

#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWindows headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif


#include "libMems/MuscleInterface.h"
#include <sstream>
#include "libGenome/gnFilter.h"
#include "libGenome/gnFASSource.h"
#include "libGenome/gnStringTools.h"

#include <fstream>

#include <wx/process.h>
#include <wx/utils.h>


boolean pipeExec( char** cmd_argv, const string& command, const string& input, string& output, string& error );

class MyProcess : public wxProcess
{
public:
	void OnTerminate( int a, int b ){ cerr << "job done\n";};

};

char** parseCommand( const string& cmd );
char** parseCommand( const string& cmd ){
	// first count tokens

	// tokenize on "
	stringstream qs( cmd );
	string cur_str;
	boolean in_quote = true;
	int token_count = 0;
	vector< string > cmd_tokens;
	while( getline( qs, cur_str, '"' ) ){
		// never start out in a quote
		in_quote = !in_quote;
		if( cur_str.length() == 0 )
			continue;
		if( in_quote ){
			cmd_tokens.push_back( cur_str );
		}else{
			stringstream ss( cur_str );
			string asdf;
			while( ss >> asdf )
				cmd_tokens.push_back( asdf );
		}
	}
	char ** cmd_array = new char*[ cmd_tokens.size() + 1 ];
	for( int tokI = 0; tokI < cmd_tokens.size(); tokI++ ){
		cmd_array[ tokI ] = new char[ cmd_tokens[ tokI ].length() + 1 ];
		strcpy( cmd_array[ tokI ], cmd_tokens[ tokI ].c_str() );
	}
	cmd_array[ cmd_tokens.size() ] = NULL;
	return cmd_array;
}

/**
 *  Wouldn't it be great to use somebody else's cross-platform pipe/execution code?
 *  Yeah, too bad it's broken on Linux.
 */
boolean wxpipeExec( const string& command, const string& input, string& output, string& error );
boolean wxpipeExec( const string& command, const string& input, string& output, string& error ){
	wxProcess* proc = new MyProcess();
	proc->Redirect();
	int pid = wxExecute( command.c_str(), 0, proc );
	if( pid == 0 ){
		delete proc;
		return false;
	}
	cerr << "muscle pid: " << pid << endl;
//	wxOutputStream* wxos = proc->GetOutputStream();
//	wxos->Write( input.c_str(), input.size() );
	proc->CloseOutput();
	
	char buf[1024];
	wxInputStream* wxis = proc->GetInputStream();
	wxInputStream* wxes = proc->GetErrorStream();
	while( true ){
		memset( buf, 0, sizeof(buf) );
		wxis->Read( buf, 1023 );
		if( wxis->LastRead() == 0 && wxis->GetLastError() == wxSTREAM_EOF )
			break;
		output += buf;
	}
	
	return true;
}

#if !defined(WIN32)
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>

// unix pipelined execution code
boolean pipeExec( char** cmd_argv, const string& command, const string& input, string& output, string& error ){
	int stdin_pipe[2], stdout_pipe[2], stderr_pipe[2];
	boolean success = false;
	pid_t sid;
	pid_t pid1;
	char* fail;
	char buf[1024];
	ssize_t bread = 0;
	int rval = 0;

	if((sid = setsid()) < 0)sid = getpgrp();
	
	if((sid < 0 && (fail = "sid"))
	 || (pipe(stdin_pipe) < 0 && (fail = "stdin"))
	 || (pipe(stdout_pipe) < 0 && (fail = "stdout"))
//	 || (pipe(stderr_pipe) < 0 && (fail = "stderr"))
	)
    {
		fprintf(stderr, "Ouch, the world just collapsed (%s).\n", fail);
		perror("muscle:");
		goto cleanup;
	}
	
	fcntl(stdin_pipe[0], F_SETFL, fcntl(stdin_pipe[0], F_GETFL) & ~O_NONBLOCK);
	fcntl(stdin_pipe[1], F_SETFL, fcntl(stdin_pipe[1], F_GETFL) & ~O_NONBLOCK);
	fcntl(stdout_pipe[0], F_SETFL, fcntl(stdout_pipe[0], F_GETFL) & ~O_NONBLOCK);
	fcntl(stdout_pipe[1], F_SETFL, fcntl(stdout_pipe[1], F_GETFL) & ~O_NONBLOCK);
/*	fcntl(stderr_pipe[0], F_SETFL, fcntl(stderr_pipe[0], F_GETFL) & ~O_NONBLOCK);
	fcntl(stderr_pipe[1], F_SETFL, fcntl(stderr_pipe[1], F_GETFL) & ~O_NONBLOCK);
*/	
	if((pid1 = fork()) < 0)goto cleanup;	
	if(pid1)
		setpgid(pid1, sid);
	else
	{
		dup2(stdin_pipe[0], 0);
		dup2(stdout_pipe[1], 1);
//		dup2(stderr_pipe[1], 2);
		close( stdin_pipe[0] );
		close( stdin_pipe[1] );
		close( stdout_pipe[0] );
		close( stdout_pipe[1] );
//		close( stderr_pipe[0] );
//		close( stderr_pipe[1] );
		execvp(cmd_argv[0], cmd_argv);
		_exit(errno);
	}
	rval = write( stdin_pipe[1], input.c_str(), input.size() );
	if( rval == -1 )
		perror( "write: " );
	if( close( stdin_pipe[1] ) )
		perror( "close stdin_w: " );
	if( close( stdin_pipe[0] ) )
		perror( "close stdin_r: " );

	close( stdout_pipe[1] );
	// read the alignment
	while(true){
		bzero( buf, sizeof(buf) );
		bread = read( stdout_pipe[0], buf, 1023 );
		if( bread == 0 )
			break;	// reached EOF
		if( bread == -1 ){
			perror("muscle read: " );
		}
		output += buf;
	}
	wait( NULL );
	success = true;
	
cleanup:
	close(stdin_pipe[0]);
	close(stdin_pipe[1]);
	close(stdout_pipe[0]);
	close(stdout_pipe[1]);
//	close(stderr_pipe[0]);
//	close(stderr_pipe[1]);
	return success;
};


#else

//windows piping code
#include <windows.h>
#define bzero(a) memset(a,0,sizeof(a)) //easier -- shortcut

bool IsWinNT()  //check if we're running NT
{
  OSVERSIONINFO osv;
  osv.dwOSVersionInfoSize = sizeof(osv);
  GetVersionEx(&osv);
  return (osv.dwPlatformId == VER_PLATFORM_WIN32_NT);
}

void ErrorMessage(char *str)  //display detailed error info
{
  LPVOID msg;
  FormatMessage(
    FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
    NULL,
    GetLastError(),
    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
    (LPTSTR) &msg,
    0,
    NULL
  );
  printf("%s: %s\n",str,msg);
  LocalFree(msg);
}

boolean pipeExec( char** cmd_argv, const string& command, const string& input, string& output, string& error ){

	char buf[1024];           //i/o buffer

	STARTUPINFO si;
	SECURITY_ATTRIBUTES sa;
	SECURITY_DESCRIPTOR sd;               //security information for pipes
	PROCESS_INFORMATION pi;
	HANDLE newstdin_w,newstdout_w,newstderr_w,newstdin_r,newstdout_r,newstderr_r;
	HANDLE read_stdout,read_stderr,write_stdin;  //pipe handles
	boolean success = false;

	if (IsWinNT())        //initialize security descriptor (Windows NT)
	{
		InitializeSecurityDescriptor(&sd,SECURITY_DESCRIPTOR_REVISION);
		SetSecurityDescriptorDacl(&sd, true, NULL, false);
		sa.lpSecurityDescriptor = &sd;
	}
	else sa.lpSecurityDescriptor = NULL;

	sa.nLength = sizeof(SECURITY_ATTRIBUTES);
	sa.bInheritHandle = true;         //allow inheritable handles

	if (!CreatePipe(&newstdin_r,&newstdin_w,&sa,0))   //create stdin pipe
	{
		ErrorMessage("CreatePipe");
		goto finito;
	}
	if (!CreatePipe(&newstdout_r,&newstdout_w,&sa,0))  //create stdout pipe
	{
		ErrorMessage("CreatePipe");
		goto finito;
	}
	if (!CreatePipe(&newstderr_r,&newstderr_w,&sa,0))  //create stdout pipe
	{
		ErrorMessage("CreatePipe");
		goto finito;
	}
	// Duplicate the write handle to the pipe so it is not inherited. 
	boolean fSuccess = DuplicateHandle(GetCurrentProcess(), newstdin_w, 
		GetCurrentProcess(), &write_stdin, 0, 
		FALSE,                  // not inherited 
		DUPLICATE_SAME_ACCESS); 
	if (! fSuccess){
		ErrorMessage("DuplicateHandle failed"); 
		goto finito;
	}
	CloseHandle(newstdin_w); 
	newstdin_w = INVALID_HANDLE_VALUE;

	// Duplicate the read handle to the pipe so it is not inherited. 
	fSuccess = DuplicateHandle(GetCurrentProcess(), newstdout_r, 
		GetCurrentProcess(), &read_stdout, 0, 
		FALSE,                  // not inherited 
		DUPLICATE_SAME_ACCESS); 
	if (! fSuccess){
		ErrorMessage("DuplicateHandle failed"); 
		goto finito;
	}
	CloseHandle(newstdout_r); 
	newstdout_r = INVALID_HANDLE_VALUE;

	// Duplicate the read handle to the pipe so it is not inherited. 
	fSuccess = DuplicateHandle(GetCurrentProcess(), newstderr_r, 
		GetCurrentProcess(), &read_stderr, 0, 
		FALSE,                  // not inherited 
		DUPLICATE_SAME_ACCESS); 
	if (! fSuccess){
		ErrorMessage("DuplicateHandle failed"); 
		goto finito;
	}
	CloseHandle(newstderr_r); 
	newstderr_r = INVALID_HANDLE_VALUE;

	GetStartupInfo(&si);      //set startupinfo for the spawned process
	/*
		The dwFlags member tells CreateProcess how to make the process.
		STARTF_USESTDHANDLES validates the hStd* members. STARTF_USESHOWWINDOW
		validates the wShowWindow member.
	*/
	si.dwFlags = STARTF_USESTDHANDLES|STARTF_USESHOWWINDOW;
	si.wShowWindow = SW_HIDE;
	si.hStdOutput = newstdout_w;
	si.hStdError = newstderr_w;     //set the new handles for the child process
	si.hStdInput = newstdin_r;

	//spawn the child process
	char* cmd = new char[ command.length() + 1 ];
	strcpy( cmd, command.c_str() );
	if (!CreateProcess(NULL,cmd,NULL,NULL,TRUE,0,
						NULL,NULL,&si,&pi))
	{
		delete cmd;
		ErrorMessage("CreateProcess");
		goto finito;
	}
	delete cmd;

	unsigned long exit=0;  //process exit code
	unsigned long bread;   //bytes read
	unsigned long avail;   //bytes available

	WriteFile(write_stdin, input.c_str(), input.size(), &bread, NULL); //send data to stdin
	CloseHandle(write_stdin);
	write_stdin = INVALID_HANDLE_VALUE;

	// Wait until child process exits.
	while( true ){
		GetExitCodeProcess( pi.hProcess, &exit );
		if( exit != STILL_ACTIVE )
			WaitForSingleObject( pi.hProcess, INFINITE );
			
		// read anything that came to stdout
		PeekNamedPipe(read_stdout,buf,1023,&bread,&avail,NULL);
		while( avail > 0 ){
			bzero(buf);
			int read_size = 1023 < avail ? 1023 : avail;
			ReadFile(read_stdout,buf,read_size,&bread,NULL);  //read the stdout pipe
			avail -= bread;
			output += buf;
		}

		// read anything that came to stderr
		PeekNamedPipe(read_stderr,buf,1023,&bread,&avail,NULL);
		while( avail > 0 ){
			bzero(buf);
			int read_size = 1023 < avail ? 1023 : avail;
			ReadFile(read_stderr,buf,read_size,&bread,NULL);  //read the stdout pipe
			avail -= bread;
			error += buf;
		}

		if( exit != STILL_ACTIVE )
			break;
	}
	success = true;

	//clean up and exit
finito:
    if( pi.hThread != INVALID_HANDLE_VALUE )
		CloseHandle(pi.hThread);
    if( pi.hProcess != INVALID_HANDLE_VALUE )
		CloseHandle(pi.hProcess);
    if( newstdin_r != INVALID_HANDLE_VALUE )
		CloseHandle(newstdin_r);
    if( newstdout_w != INVALID_HANDLE_VALUE )
		CloseHandle(newstdout_w);
    if( newstderr_w != INVALID_HANDLE_VALUE )
		CloseHandle(newstderr_w);
    if( read_stdout != INVALID_HANDLE_VALUE )
		CloseHandle(read_stdout);
    if( read_stderr != INVALID_HANDLE_VALUE )
		CloseHandle(read_stderr);
    if( write_stdin != INVALID_HANDLE_VALUE )
		CloseHandle(write_stdin);
	return success;
}

#endif



MuscleInterface& MuscleInterface::getMuscleInterface()
{
        static MuscleInterface m_ci;

        return m_ci;
}

MuscleInterface::MuscleInterface(){
	muscle_path = "muscle";
	muscle_arguments = "-stable -quiet";
	muscle_cmdline = parseCommand( muscle_path + " " + muscle_arguments );
}

void MuscleInterface::ParseMusclePath( const char* argv0 ){
	// get the execution path
	string path_str = argv0;
	// trim quotes
	if( path_str[0] == '"' )
		path_str = path_str.substr( 1, path_str.size() - 2 );
	standardizePathString( path_str );
	string::size_type i = path_str.rfind('/');
	if( i != string::npos )
		path_str.erase(i+1, path_str.length() - (i+1));
	else
		path_str.clear();
	SetMusclePath( '"' + path_str + "muscle\"");
}

void MuscleInterface::SetMusclePath( const string& path ){
	muscle_path = path;
	muscle_cmdline = parseCommand( muscle_path + " " + muscle_arguments );
}

void MuscleInterface::SetMuscleArguments( const string& args ){
	muscle_arguments = args;
	muscle_cmdline = parseCommand( muscle_path + " " + muscle_arguments );
}

MuscleInterface& MuscleInterface::operator=( const MuscleInterface& ci ){
	return *this;
}

boolean MuscleInterface::Align( GappedAlignment& cr, Match* r_begin, Match* r_end, vector< gnSequence* >& seq_table ){
	int max_muscle_length = 100000;
	gnSeqI gap_size = 0;
	boolean create_ok = true;
	uint seq_count = seq_table.size();
	uint seqI;
	uint align_seqs = 0;
try{

// 
//	Get the sequence in the intervening gaps between these two matches
//
	vector< string > seq_data;
	vector< int64 > starts;
	vector< uint > seqs;
	const gnFilter* rc_filter = gnFilter::DNAComplementFilter();
	
	for( seqI = 0; seqI < seq_count; seqI++ ){

		// skip this sequence if it's undefined
		if( (r_end != NULL && r_end->Start( seqI ) == MEM_NO_MATCH ) ||
			(r_begin != NULL && r_begin->Start( seqI ) == MEM_NO_MATCH) ){
			starts.push_back( MEM_NO_MATCH );
			continue;
		}

		// determine the size of the gap
		int64 gap_start = 0;
		int64 gap_end = 0;
		getInterveningCoordinates( seq_table, r_begin, r_end, seqI, gap_start, gap_end );

		int64 diff = gap_end - gap_start;
		if( diff <= 0 || diff > max_muscle_length ){
			starts.push_back( MEM_NO_MATCH );
			continue;	// skip this sequence if it's either too big or too small
		}
		seqs.push_back( seqI );

		// extract sequence data
		if( r_end == NULL || r_end->Start( seqI ) > 0 ){
			starts.push_back( gap_start );
			seq_data.push_back( seq_table[ seqI ]->ToString( diff , gap_start ) );
		}else{
			// reverse complement the sequence data.
			starts.push_back( -gap_start );
			string cur_seq_data = seq_table[ seqI ]->ToString( diff , gap_start );
			rc_filter->ReverseFilter( cur_seq_data );
			seq_data.push_back( cur_seq_data );
		}
	}

	if( seqs.size() <= 1 )
		create_ok = false;

	if( create_ok ){
		vector< string > aln_matrix;
		if( !CallMuscle( aln_matrix, seq_data ) ){
			cout << "Muscle was unable to align:\n";
			if( r_begin )
				cout << "Left match: " << *r_begin << endl;
			if( r_end )
				cout << "Right match: " << *r_end << endl;
			return false;
		}

		gnSeqI aln_length = aln_matrix.size() == 0 ? 0 : aln_matrix[0].length();
		cr = GappedAlignment( seq_count, aln_length );
		if( aln_matrix.size() > 0 ){
			vector< string > tmp_mat = vector< string >( seq_count );
			cr.setAlignment( tmp_mat, 0 );
		}
		cr.align_length = aln_length;
		// set sequence starts
		for( uint seqI = 0; seqI < seqs.size(); seqI++ ){
			cr.SetLength( seq_data[ seqs[ seqI ] ].size(), seqs[ seqI ] );
			cr.align_matrix[ seqs[ seqI ] ] = aln_matrix[ seqI ];
		}
		for( uint seqI = 0; seqI < cr.align_matrix.size(); seqI++ ){
			cr.SetStart( seqI, starts[ seqI ] );
			if( cr.align_matrix[ seqI ].length() != aln_length )
				cr.align_matrix[ seqI ] = string( aln_length, '-' );
		}
		return true;
	}
}catch(exception& e){
	cerr << "At: " << __FILE__ << ":" << __LINE__ << endl;
	cerr << e.what();
}
	return false;
}


boolean MuscleInterface::CallMuscle( vector< string >& aln_matrix, const vector< string >& seq_table ){

	try{
		ostringstream input_seq_stream;
		gnSequence seq;
		for( uint seqI = 0; seqI < seq_table.size(); seqI++ ){
			seq += seq_table[ seqI ];
			seq.setContigName( seqI, "seq" );
		}
		gnFASSource::Write( seq, input_seq_stream, false, true );
		
		// now open a pipe to Muscle
		string muscle_cmd = muscle_path + " " + muscle_arguments;
		string output;
		string error;
		boolean success = pipeExec( muscle_cmdline, muscle_cmd, input_seq_stream.str(), output, error );
		if( !success || output.size() == 0 )
			return false;

		istringstream output_aln_stream( output );
		string cur_line;

		// parse the fasta output
		while( getline( output_aln_stream, cur_line ) ){
			if( cur_line[0] == '>' ){
				aln_matrix.push_back( "" );
				continue;
			}
			gnSeqI len = cur_line.size();
			len = cur_line[ len - 1 ] == '\r' ? len - 1 : len;
			uint seqI = aln_matrix.size() - 1;
			aln_matrix[ seqI ] += cur_line.substr( 0, len );
		}

		return true;
	}catch( gnException& gne ){
	}catch( exception& e ){
	}catch(...){
	}
	return false;
}
